@@ -297,7 +297,6 @@ sub launch_server {
    my $configure_server=$_[5]||
          sub { print "NO configure_server method defined!" };
    my $tag=$_[6]||'';
-   my $region=$_[7]||'';
    my $num=$cnt+1;
    my ($server_host_block,$handle,$hash,$output,$error,$inst)=('','','','','');
    foreach (0..2) {
@@ -331,7 +330,7 @@ sub launch_server {
    $main::aws->{$server_type}->[$cnt]->[1]=$handle;
    my ($stdout,$stderr,$exitcode)=('','','');
    $configure_server->($server_type,$cnt,$selection,$server_host_block,
-                       $region);
+                       @_[7..$#_]);
 }
 
 our $aws_configure=sub {
