@@ -50,6 +50,14 @@ my $configure_gnusocial=sub {
    my $selection=$_[2]||'';
    my $region=$_[4]||'';
    my $verified_email=$_[5]||'';
+   my $permanent_ip=$_[6]||'';
+   $permanent_ip='' if $permanent_ip=~/Stay/;
+   if ($permanent_ip=~/Elastic/) {
+      my $c="aws ec2 allocate-address --domain vpc";
+      my ($hash,$output,$error)=run_aws_cmd($c);
+      $hash||={};$hash->{Addresses}||=[];
+      $permanent_ip=$hash->{Addresses}->{PublicIp}; 
+   }
    my $handle=$main::aws->{$server_type}->[$cnt]->[1];
    my ($stdout,$stderr)=('','');
    ($stdout,$stderr)=$handle->cmd("sudo yum -y update",'__display__');
@@ -150,6 +158,38 @@ print "\n\n\n\n\n\n\nWE SHOULD HAVE INSTALLED MARIADB=$stdout<==\n\n\n\n\n\n\n";
       $public_ip=$addr if $addr=~/^\d+\.\d+\.\d+\.\d+\s*/s;
    }
    chomp($public_ip);
+   if ($public_ip && $permanent_ip) {
+      my $c="aws ec2 describe-instances";
+      my ($hash,$output,$error)=run_aws_cmd($c);
+      $hash||={};
+      $c="aws ec2 describe-addresses";
+      my ($hasha,$outputa,$errora)=run_aws_cmd($c);
+      $hasha||={};$hasha->{Addresses}||=[];
+      my $a_id='';
+      foreach my $address (@{$hasha->{Addresses}}) {
+         if ($permanent_ip eq $address->{PublicIp}) {
+            $a_id=$address->{AllocationId};
+            last;
+         }
+      }
+      my %pubip=();my $instance_id='';
+      foreach my $res (@{$hash->{Reservations}}) {
+         foreach my $inst (@{$res->{Instances}}) {
+            my $pip=$inst->{PublicIpAddress}||'';
+            my $iid=$inst->{InstanceId}||'';
+            next if exists $inst->{State}->{Name} &&
+               $inst->{State}->{Name} eq 'terminated';
+            if ($public_ip eq $pip) {
+               my $c="aws ec2 associate-address --instance-id ".
+                     $inst->{InstanceId}." --allocation-id $a_id ".
+                     "--allow-reassociation";
+               my ($hasha,$outputa,$errora)=run_aws_cmd($c);
+               $public_ip=$permanent_ip;
+               last;
+            }
+         }
+      } 
+   }
    $public_ip='localhost' unless $public_ip;
    my $sudo='sudo ';
    # https://nealpoole.com/blog/2011/04/setting-up-php-fastcgi-and-nginx
@@ -163,6 +203,7 @@ print "\n\n\n\n\n\n\nWE SHOULD HAVE INSTALLED MARIADB=$stdout<==\n\n\n\n\n\n\n";
    # https://wiki.loadaverage.org/gnusocial/installation_guides/install_like_loadaverage
    # https://karp.id.au/social/index.html
    # http://jeffreifman.com/how-to-install-your-own-private-e-mail-server-in-the-amazon-cloud-aws/
+   # http://www.linuxveda.com/2015/06/05/gnu-social-vs-twitter/
    my $nginx='nginx-1.10.0';
    $nginx='nginx-1.9.13' if $^O eq 'cygwin';
    ($stdout,$stderr)=$handle->cmd("sudo wget --random-wait --progress=dot ".
@@ -713,7 +754,7 @@ END
          }
       }
    }
-print "\nSMTP SERVER = $smtp_server and VERIFIED_EMAIL = $verified_email\n";
+print "\nSMTP SERVER = $smtp_server and\nVERIFIED_EMAIL = $verified_email and\nSMTPPASS=$smtppass and\nID=$hash->{AccessKey}->{AccessKeyId}\n";
    print "\n   ACCESS GNU SOCIAL UI AT:\n\n",
          " https://$public_ip/index.php\n";
    my $thanks=<<'END';
@@ -761,6 +802,7 @@ my $standup_gnusocial=sub {
       $verified_email="]I[{'gnusocial_enter_email_address',1}";
    }
    my $gnusocial="]T[{select_gnusocial_setup}";
+   my $permanent_ip="]T[{permanent_ip}";
    my $i=$main::aws->{fullauto}->{ImageId}||'';
    my $s=$main::aws->{fullauto}->
          {NetworkInterfaces}->[0]->{SubnetId}||'';
@@ -809,12 +851,14 @@ my $standup_gnusocial=sub {
          "--security-group-ids $g --subnet-id $s";
       if ($#{$main::aws->{'GNU.io/Social'}}==0) {
          launch_server('GNU.io/Social',$cnt,$gnusocial,'',$c,
-         $configure_gnusocial,'',$region,$verified_email);
+         $configure_gnusocial,'',$region,$verified_email,
+         $permanent_ip);
       } else {
          my $num=$#{$main::aws->{'GNU.io/Social'}}-1;
          foreach my $num (0..$num) {
             launch_server('GNU.io/Social',$cnt++,$gnusocial,'',$c,
-            $configure_gnusocial,'',$region,$verified_email);
+            $configure_gnusocial,'',$region,$verified_email,
+            $permanent_ip);
          }
       }
    }
@@ -839,8 +883,25 @@ my $gnusocial_setup_summary=sub {
    my $gnusocial="]T[{select_gnusocial_setup}";
    $gnusocial=~s/^"//;
    $gnusocial=~s/"$//;
-   print "REGION=$region and TYPE=$type\n";
-   print "GNUSOCIAL=$gnusocial\n";
+   my $permanent_ip="]T[{permanent_ip}";
+   if ($permanent_ip=~/^["]Release.* (\d+\.\d+\.\d+\.\d+).*$/s) {
+      my $ip_to_release=$1;
+      my $c="aws ec2 describe-addresses";
+      my ($hash,$output,$error)=
+         &Net::FullAuto::Cloud::fa_amazon::run_aws_cmd($c);
+      $hash||={};$hash->{Addresses}||=[];
+      foreach my $address (@{$hash->{Addresses}}) {
+         if ($address->{PublicIp} eq $ip_to_release) {
+            my $c="aws ec2 release-address ".
+                  "--allocation-id $address->{AllocationId}";
+            my ($hash,$output,$error)=
+               &Net::FullAuto::Cloud::fa_amazon::run_aws_cmd($c);
+            last;
+         }
+      }
+      print "\n   $ip_to_release HAS BEEN RELEASED . . .\n";
+      sleep 5;
+   }
    my $num_of_servers=0;
    my $ol=$gnusocial;
    $ol=~s/^.*(\d+)\sServer.*$/$1/;
@@ -913,6 +974,164 @@ END
 
 };
 
+our $check_elastic_ip=sub {
+
+   my $c="aws ec2 describe-addresses";
+   my ($hash,$output,$error)=run_aws_cmd($c);
+   $hash||={};$hash->{Addresses}||=[];
+   if (-1==$#{$hash->{Addresses}}) {
+      my $permanent_ip_banner=<<'END';
+    ___                                 _     ___ ___ ___
+   | _ \___ _ _ _ __  __ _ _ _  ___ _ _| |_  |_ _| _ \__ \
+   |  _/ -_) '_| '  \/ _` | ' \/ -_) ' \  _|  | ||  _/ /_/
+   |_| \___|_| |_|_|_\__,_|_||_\___|_||_\__| |___|_|  (_)
+END
+      $permanent_ip_banner.=<<END;
+
+   If you plan on using this GNU Social server for an extended
+   period, you will need a non-temporary IP Address. In Amazon
+   Web Services, one Elastic IP (Amazon's way of allocating long 
+   term IP addresses) is free so long as it is associated to
+   a running server. If you stop or terminate the server, you
+   need to manually release the address, or you will incur charges.
+
+   Note also, that a Permanent IP Address will substantially
+   increase the likelihood of email being sent from the GNU Social
+   server surviving spam filters and actually arriving at the
+   intended destination.
+
+END
+      my $permanent_ip={
+
+         Name => 'permanent_ip',
+         Item_1 => {
+
+            Text => 'Use Elastic (Permanent) IP Address',
+            Result => $gnusocial_setup_summary,
+
+         },
+         Item_2 => {
+
+            Text => 'Stay with Temporary Public IP Address',
+            Result => $gnusocial_setup_summary,
+
+         },
+         Scroll => 2,
+         Result => $gnusocial_setup_summary,
+         Banner => $permanent_ip_banner,
+
+      };
+      return $permanent_ip;
+   } else {
+      my $c="aws ec2 describe-instances";
+      my ($hash,$output,$error)=run_aws_cmd($c);
+      $hash||={};$hash->{Addresses}||=[];
+      my %pubip=();
+      foreach my $res (@{$hash->{Reservations}}) {
+         foreach my $inst (@{$res->{Instances}}) {
+            my $pip=$inst->{PublicIpAddress}||'';
+            next unless $pip;
+            next if exists $inst->{State}->{Name} &&
+               $inst->{State}->{Name} eq 'terminated';
+            $pubip{$pip}='';
+         }
+      }
+      $c="aws ec2 describe-addresses";
+      ($hash,$output,$error)=run_aws_cmd($c);
+      my @available=();my @available_remove=();
+      foreach my $address (@{$hash->{Addresses}}) {
+         unless (exists $pubip{$address->{PublicIp}}) {
+            push @available, $address->{PublicIp};
+            push @available_remove,"Release IP Address $address->{PublicIp}"; 
+         }
+      }
+      if (-1<$#available) {
+         my $use_elastic_banner=<<'END';
+    _   _           ___ _         _   _      ___ ___ ___ 
+   | | | |___ ___  | __| |__ _ __| |_(_)__  |_ _| _ \__ \ 
+   | |_| (_-</ -_) | _|| / _` (_-<  _| / _|  | ||  _/ /_/
+    \___//__/\___| |___|_\__,_/__/\__|_\__| |___|_|  (_)
+END
+         $use_elastic_banner.=<<END;
+
+   An allocated but not associated Elastic IP has been identified.
+   Using this IP Address for GNU Social may avoid charges from
+   Amazon that are levied against allocated but not associated
+   Elastic IPs. Please make a selection.
+
+END
+         my $permanent_ip={
+
+            Name => 'permanent_ip',
+            Item_1 => {
+
+               Text => "Use Elastic (Permanent) IP Address ]C[",
+               Convey => \@available,
+               Result => $gnusocial_setup_summary,
+
+            },
+            Item_2 => {
+
+               Text => "Stay with Temporary Public IP Address &\n".
+                       "                 ]C[ (to avoid cost)",
+               Convey => \@available_remove,
+               Result => $gnusocial_setup_summary,
+
+            },
+            Item_3 => {
+
+               Text => 'Stay with Temporary Public IP Address',
+               Result => $gnusocial_setup_summary,
+
+            },
+            Scroll => 3,
+            Result => $gnusocial_setup_summary,
+            Banner => $use_elastic_banner,
+
+         };
+         return $permanent_ip;
+      } else {
+         my $new_elastic_banner=<<'END';
+    _  _              ___ _         _   _      ___ ___ ___
+   | \| |_____ __ __ | __| |__ _ __| |_(_)__  |_ _| _ \__ \
+   | .` / -_) V  V / | _|| / _` (_-<  _| / _|  | ||  _/ /_/
+   |_|\_\___|\_/\_/  |___|_\__,_/__/\__|_\__| |___|_|  (_)
+END
+         $new_elastic_banner.=<<END;
+
+   Allocated Elastic IP Addresses have been identified, but
+   all are currently associated with server instances. A new
+   one can be allocated and associated with your new GNU
+   Social server, but an additional cost of \$0.005 (half cent)
+   per hour will be incurred. Please make a selection.
+
+END
+         my $permanent_ip={
+
+            Name => 'permanent_ip',
+            Item_1 => {
+
+               Text => 'Allocate Additional Elastic (Permanent) IP Address',
+               Result => $gnusocial_setup_summary,
+
+            },
+            Item_2 => {
+
+               Text => 'Stay with Temporary Public IP Address',
+               Result => $gnusocial_setup_summary,
+
+            },
+            Scroll => 2,
+            Result => $gnusocial_setup_summary,
+            Banner => $new_elastic_banner,
+
+         };
+         return $permanent_ip;
+      }
+   }
+
+};
+
 our $gnusocial_validate_email=sub {
 
    my $email="]I[{'gnusocial_enter_email_address',1}";
@@ -922,16 +1141,16 @@ our $gnusocial_validate_email=sub {
       print "\n   ERROR: Email Addresses do not match!";sleep 5;
       STDOUT->autoflush(0);
       return '<';
-   } elsif ($email!~/\@/) {
-      STDOUT->autoflush(1);
-      print "\n   ERROR: Email Address must contain '@' character!";sleep 5;
-      STDOUT->autoflush(0);
-      return '<';
    } elsif ($email=~/^\s*$/) {
       STDOUT->autoflush(1);
       print "\n   ERROR: You failed to enter an Email Address!";sleep 5;
       STDOUT->autoflush(0);
       return '<';
+   } elsif ($email!~/\@/) {
+      STDOUT->autoflush(1);
+      print "\n   ERROR: Email Address must contain \'\@\' character!";sleep 5;
+      STDOUT->autoflush(0);
+      return '<';
    }
    my $c="aws ses verify-email-identity --email-address $email";
    my ($hash,$output,$error)=run_aws_cmd($c);
@@ -958,7 +1177,7 @@ END
    my $email_message={
 
       Name => 'email_message',
-      Result => $gnusocial_setup_summary,
+      Result => $check_elastic_ip,
       Banner => $inform_banner,
 
    };
@@ -1050,9 +1269,10 @@ END
 
             Text => "]C[",
             Convey => \@identities,
-            Result => $gnusocial_setup_summary,
+            Result => $check_elastic_ip,
 
-         },    
+         },
+         Scroll => 2,
          Banner => $pick_banner,
       };
       return $pick_email;
